import json

import listpad
import numpy as np
import tensorflow as tf
import qnd

from .. import classify
from .nlp import *


__all__ = ['def_read_file']


def def_convert_json_example():
    qnd.add_flag('document_length', type=int, default=32)
    qnd.add_flag('sentence_length', type=int, default=64)
    add_word_file_flag()
    classify.add_num_classes_flag()

    def convert_json_example(string):
        word_indices = {word: index for index,
                        word in enumerate(qnd.FLAGS.words)}

        def convert(string):
            example = json.loads(string.decode())

            document = example['document']
            label = example['label']['binary'
                                     if qnd.FLAGS.num_classes == 2 else
                                     'multi']

            return tuple(map(
                lambda x: np.array(x, np.int32),
                [listpad.ListPadder(
                    [qnd.FLAGS.sentence_length, qnd.FLAGS.document_length],
                    NULL_INDEX)
                 .pad([[(word_indices[word]
                         if word in word_indices else
                         UNKNOWN_INDEX)
                        for word in sentence]
                       for sentence in document]),
                 label,
                 len(document)]))

        document, label, document_length = tf.py_func(
            convert,
            [string],
            [tf.int32, tf.int32, tf.int32],
            name="convert_json_example")

        label.set_shape([])
        document_length.set_shape([])

        return (tf.reshape(document,
                           [qnd.FLAGS.document_length,
                            qnd.FLAGS.sentence_length]),
                label)

    return convert_json_example


def def_read_file():
    convert_json_example = def_convert_json_example()

    def read_file(filename_queue):
        _key, value = tf.WholeFileReader().read(filename_queue)
        return convert_json_example(value)

    return read_file



import argtyp
import extenteten as ex
import tensorflow as tf
import qnd


def add_num_classes_flag():
    qnd.add_required_flag("num_classes", type=int)


def def_classify():
    add_num_classes_flag()
    qnd.add_flag("num_labels", type=int, help="Required only for inference")
    qnd.add_flag("hidden_layer_sizes", type=argtyp.int_list, default=[100])
    qnd.add_flag("dropout_keep_prob", type=float)

    def classify(feature, label=None, *, mode, regularization_scale=1e-8):
        if qnd.FLAGS.num_classes <= 1:
            raise ValueError("Number of classes must be greater than 1.")

        num_labels = qnd.FLAGS.num_labels or ex.num_labels(label)

        return ex.classify(
            ex.mlp(
                feature,
                layer_sizes=[
                    *qnd.FLAGS.hidden_layer_sizes,
                    ex.num_logits(num_labels, qnd.FLAGS.num_classes)],
                dropout_keep_prob=(
                    qnd.FLAGS.dropout_keep_prob
                    if mode == tf.contrib.learn.ModeKeys.TRAIN else
                    None)),
            label,
            num_classes=qnd.FLAGS.num_classes,
            num_labels=num_labels,
            regularization_scale=regularization_scale)

    return classify



import re
import setuptools
import sys


if not sys.version_info >= (3, 5):
    exit("Sorry, Python must be later than 3.5.")


setuptools.setup(
    name="tensorflow-qndex",
    version=re.search(r'__version__ *= *"([0-9]+\.[0-9]+\.[0-9]+)" *\n',
                      open("qndex/__init__.py").read()).group(1),
    description="tensorflow-qnd x tensorflow-extenteten",
    long_description=open("README.md").read(),
    license="Public Domain",
    author="Yota Toyama",
    author_email="raviqqe@gmail.com",
    url="https://github.com/raviqqe/tensorflow-qndex/",
    packages=["qndex", "qndex.nlp"],
    install_requires=[
        "tensorflow-qnd",
        "tensorflow-extenteten",
        "argtyp",
        "listpad",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: Public Domain",
        "Programming Language :: Python :: 3.5",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: System :: Networking",
    ],
)
